import numpy as np
import datetime
import matplotlib.pyplot as plt
from matplotlib import pyplot
import matplotlib.dates as dates
import pandas as pd
from pandas import read_csv
from pandas.tseries.frequencies import to_offset
from datetime import datetime
from scipy import optimize
import math as m
from numpy.linalg import pinv  # for matrix inversion! 


header_list = ['Year','Month','Day','Hour','Minute','Second','Pressure']  # names of columns
mat = pd.read_csv('pressure_Bern_2021.csv', header=1, parse_dates=True, names=header_list)

##      Year  Month  Day  Hour  Minute  Second  Pressure
##0     2021      6    1     0      10       0    949.87
##1     2021      6    1     0      20       0    949.90
##2     2021      6    1     0      30       0    949.99


# pressure series p 
p=mat['Pressure'].values  # in unit hectopascal: 1 hPa=100 Pa


ut=mat['Hour'].values  # Universal Time UT (hour)

# Get the indices of elements with certain ut values
x=np.arange(0,24,1)  # make an integer array 0,1,2 ...
pm=np.zeros(24)   #  make a float array with zeros
for i in x:  #  for loop (the next command lines should be a bit shifted to the right hand side)
    ind = np.where(ut == i)    # find the indices for each hour of the complete series 
    pm[i]  = np.nanmean(p[ind])   # compute the mean of p at the indices

# pm is the daily cycle of pressure at Bern averaged from June to August

x=x+.5   # shift the time vector to the center of the averaged time intervals


######################################################################################
# Decomposition of the observed daily cycle (pm) into the diurnal and semidiurnal tide
# We are using the relation:
#       am sin(omega*x + phi ) = a sin(omega*x) + b cos(omega*x)
# It is easier to work with the right side of the equation since it is a linear combination.
# If a and b are derived, we easily get am and phi: 
# am=sqrt(a**2 +b**2)
# phi=arctan2(b,a)
#####################################################################################

n=len(x)
v1=np.zeros(n)
v2=np.zeros(n)
v3=np.zeros(n)
v4=np.zeros(n)
v5=np.zeros(n)

om1=2*m.pi/24 # frequency omega of diurnal tide
om2=2*m.pi/12 # semidiurnal
ind=np.arange(0,n)
for i in ind:                  # create 5 Eigenvectors for decomposition
    v1[i] = 1.
    v2[i] = m.sin(om1*x[i])
    v3[i] = m.cos(om1*x[i])
    v4[i] = m.sin(om2*x[i])
    v5[i] = m.cos(om2*x[i])



ncol=5   # 5 Eigenvectors for 5 unknowns (c,a1,b1,a2,b2)
nrow=n
A = np.zeros((nrow, ncol), dtype=np.float64)  # making the matrix A  
A[:,0]=v1
A[:,1]=v2
A[:,2]=v3
A[:,3]=v4
A[:,4]=v5

# we have the equation system:  A * (c,a1,b1,a2,b2) = pm  
# or: pm = c*v1 + a1*v2 + b1*v3 + a2*v4 + b2*v5 

a = pinv(A)  # most difficult task: inversion of the matrix A. Then we get: (c,a1,b1,a2,b2)= a * pm


# pm= 500 + 0*v2 + 1*v3 + 0*v4 + 1*v5  #  if you like to test with an artificial signal series 

result= a.dot(pm)   # matrix multiplication with vector pm gives "result" vector [c, a1, b1, a2, b2] 
c=result[0]
a1=result[1]
b1=result[2]
a2=result[3]
b2=result[4]

pfit= c + a1*v2 + b1*v3 + a2*v4 + b2*v5 # superposition of diurnal and semidiurnal
pfit1= c + a1*v2 + b1*v3          # diurnal component (plus constant)
pfit2= c + a2*v4 + b2*v5          # semidiurnal component 

# amplitudes of the tides
am1=m.sqrt(a1**2+ b1**2)  # as already explained above 
am2=m.sqrt(a2**2+ b2**2) 

# phases of tides (time of maximum )
phi1= np.arctan2(b1, a1)
print(phi1*180/m.pi) # phase (deg)
phi1=(m.pi/2-phi1)/om1  # time of maximum
phi2= np.arctan2(b2, a2)
print(phi2*180/m.pi)
phi2=(m.pi/2-phi2)/om2 + 12  # time of maximum  (one can add the period 12 hours to get a positive value) 


print('diurnal amplitude and phase (time of maximum):' , np.round(am1,decimals=2), 'hPa, phase:',np.round(phi1,decimals=2),'UT')
print('semidiurnal amplitude and phase (time of maximum):' ,np.round(am2,decimals=2), 'hPa, phase:',np.round(phi2,decimals=2),'UT') 
print('relative amplitudes of diurnal and semidiurnal tides:',np.round(am1/c*100,decimals=3), '% and ',np.round(am2/c*100,decimals=3),'%'   )
print('These are really weak but reliable signals!')

fig, ax = plt.subplots()
ax.plot(x, pm, label='observed')
ax.plot(x,pfit, label='superposition of components')
ax.plot(x,pfit1,label='diurnal component')
ax.plot(x,pfit2, label='semidiurnal component')
ax.set_xlabel('Universal Time (hour)')
ax.set_ylabel('Pressure (hPa)')
ax.legend()

plt.savefig('p_decomposition.pdf')
plt.show()
